@@ -1,7 +1,7 @@
 # ============================================================
 #
 # Copyright (C) 2010 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
-# Copyright (C) 2011, 2012 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
+# Copyright (C) 2011, 2012, 2013 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 #
 # This file may be licensed under the terms of the
 # GNU Lesser General Public License Version 3 (the ``LGPL''),
@@ -166,42 +166,18 @@ class ParticipantConfig (object):
 
         @author: jmoringe
         """
-        def __init__(self, name, options={}, converter_ = None):
-            import rsb.converter
+        def __init__(self, name, options={}, converters=None):
             self.__name = name
             self.__enabled = options.get('enabled', '0') in ('1', 'true', 'yes')
-            # Obtain a consistent converter set for the wire-type of
-            # the transport:
-            # 1. Find global converter map for the wire-type
-            # 2. Find configuration options that specify converters
-            #    for the transport
-            # 3. Add converters from the global map to the unambiguous
-            #    map of the transport, resolving conflicts based on
-            #    configuration options when necessary
-            wireType = bytearray
-
-            if not converter_:
-                self.__converters = rsb.converter.UnambiguousConverterMap(wireType)
-                # Find and transform configuration options
-                converterOptions = dict([ (key[17:], value) for (key, value) in options.items()
-                                          if key.startswith('converter.python') ])
-                # Try to add converters form global map
-                globalMap = rsb.converter.getGlobalConverterMap(wireType)
-                for ((wireSchema, dataType), converter) in globalMap.getConverters().items():
-                    # Converter can be added if converterOptions does not
-                    # contain a disambiguation that gives precedence to a
-                    # different converter. map may still raise an
-                    # exception in case of ambiguity.
-                    if not wireSchema in converterOptions \
-                            or dataType.__name__ == converterOptions[wireSchema]:
-                        self.__converters.addConverter(converter)
-            else:
-                # If the converter has already been created and configured.
-                self.__converters = converter_
 
             # Extract freestyle options for the transport.
             self.__options = dict([ (key, value) for (key, value) in options.items()
                                    if not '.' in key and not key == 'enabled' ])
+            # Find converter selection rules
+            self.__converters = converters
+            self.__converterRules \
+                = dict([ (key[len("converter.python."):], value) for (key, value) in options.items()
+                         if key.startswith('converter.python') ])
 
         def getName(self):
             return self.__name
@@ -219,7 +195,18 @@ class ParticipantConfig (object):
         def getConverters(self):
             return self.__converters
 
-        converters = property(getConverters)
+        def setConverters(self, converters):
+            self.__converters = converters
+
+        converters = property(getConverters, setConverters)
+
+        def getConverterRules(self):
+            return self.__converterRules
+
+        def setConverterRules(self, converterRules):
+            self.__converterRules = converterRules
+
+        converterRules = property(getConverterRules, setConverterRules)
 
         def getOptions(self):
             return self.__options
@@ -227,8 +214,8 @@ class ParticipantConfig (object):
         options = property(getOptions)
 
         def __str__(self):
-            return ('ParticipantConfig.Transport[%s, enabled = %s,  converters = %s, options = %s]'
-                    % (self.__name, self.__enabled, self.__converters, self.__options))
+            return ('ParticipantConfig.Transport[%s, enabled = %s,  converters = %s, converterRules = %s, options = %s]'
+                    % (self.__name, self.__enabled, self.__converters, self.__converterRules, self.__options))
 
         def __repr__(self):
             return str(self)
@@ -330,7 +317,7 @@ class ParticipantConfig (object):
         for (key, value) in os.environ.items():
             if key.startswith('RSB_'):
                 if value == '':
-                   raise ValueError, 'The value of the environment variable %s is the empty string' % key
+                    raise ValueError, 'The value of the environment variable %s is the empty string' % key
                 options[key[4:].lower().replace('_', '.')] = value
         return options
 
@@ -389,6 +376,55 @@ class ParticipantConfig (object):
         options = clazz.__fromEnvironment(partial)
         return clazz.__fromDict(options)
 
+def convertersFromTransportConfig(transport):
+    """
+    Returns an object implementing the
+    L{rsb.converter.ConverterSelectionStrategy} protocol suitable for
+    B{transport}.
+
+    If C{transport.converters} is not C{None}, it is used
+    unmodified. Otherwise the specification in
+    C{transport.converterRules} is used.
+
+    @return: The constructed ConverterSelectionStrategy object.
+    @rtype: ConverterSelectionStrategy
+    """
+
+    # There are two possible ways to configure converters:
+    # 1) transport.converters: this is either None or an object
+    #    implementing the "ConverterSelectionStrategy protocol"
+    # 2) when transport.converters is None, transport.converterRules
+    #    is used to construct an object implementing the
+    #    "ConverterSelectionStrategy protocol"
+    if transport.converters is not None:
+        return transport.converters
+
+    # Obtain a consistent converter set for the wire-type of
+    # the transport:
+    # 1. Find global converter map for the wire-type
+    # 2. Find configuration options that specify converters
+    #    for the transport
+    # 3. Add converters from the global map to the unambiguous map of
+    #    the transport, resolving conflicts based on configuration
+    #    options when necessary
+    # TODO hack!
+    wireType = bytearray
+
+    import rsb
+    import rsb.converter
+    converterMap = rsb.converter.UnambiguousConverterMap(wireType)
+    # Try to add converters form global map
+    globalMap = rsb.converter.getGlobalConverterMap(wireType)
+    for ((wireSchema, dataType), converter) in globalMap.getConverters().items():
+        # Converter can be added if converterOptions does not
+        # contain a disambiguation that gives precedence to a
+        # different converter. map may still raise an
+        # exception in case of ambiguity.
+        if not wireSchema in transport.converterRules \
+           or dataType.__name__ == transport.converterRules[wireSchema]:
+            converterMap.addConverter(converter)
+    return converterMap
+
 class Scope(object):
     """
     A scope defines a channel of the hierarchical unified bus covered by RSB.
@@ -561,6 +597,9 @@ class Scope(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return hash(self.toString())
+
     def __lt__(self, other):
         return self.toString() < other.toString()
 
@@ -825,7 +864,7 @@ class Event(object):
         @type id: EventId
         @param scope: A L{Scope} designating the channel on which the
                       event will be published.
-        @type scope: Scope
+        @type scope: Scope or accepted by Scope constructor
         @param method: A string designating the "method category"
                        which identifies the role of the event in some
                        communication patters. Examples are
@@ -1061,9 +1100,10 @@ class Event(object):
     causes = property(getCauses, setCauses)
 
     def __str__(self):
-        printData = self.__data
-        if isinstance(self.__data, str) and len(self.__data) > 10000:
-            printData = "string with length %u" % len(self.__data)
+        printData = str(self.__data)
+        if len(printData) > 100:
+            printData = printData[:100] + '...'
+        printData = ''.join(['\\x%x' % ord(c) if ord(c) < 32 else c for c in printData])
         return "%s[id = %s, scope = '%s', data = '%s', type = '%s', metaData = %s, causes = %s]" \
             % ("Event", self.__id, self.__scope, printData, self.__type, self.__metaData, self.__causes)
 
@@ -1154,7 +1194,7 @@ class Participant(object):
                     assert(False)
             else:
                 raise ValueError, 'No such transport: "%s"' % transport.getName()
-            transports.append(klass(converters = transport.getConverters(),
+            transports.append(klass(converters = convertersFromTransportConfig(transport),
                                     options    = transport.getOptions()))
         return transports
 
@@ -1173,7 +1213,7 @@ class Informer(Participant):
         payloads of type B{type} on B{scope}.
 
         @param scope: scope of the informer
-        @type scope: Scope
+        @type scope: Scope or accepted by Scope constructor
         @param theType: A Python object designating the type of objects
                         that will be sent via the new informer. Instances
                         of subtypes are permitted as well.
@@ -1227,7 +1267,7 @@ class Informer(Participant):
 
     def publishData(self, data, userInfos=None, userTimes=None):
         # TODO check activation
-        self.__logger.debug("Publishing data '%s'" % data)
+        self.__logger.debug("Publishing data '%s'", data)
         event = Event(scope = self.scope,
                       data = data, type = type(data),
                       userInfos = userInfos, userTimes = userTimes)
@@ -1256,7 +1296,7 @@ class Informer(Participant):
         with self.__mutex:
             event.id = EventId(self.id, self.__sequenceNumber)
             self.__sequenceNumber += 1
-        self.__logger.debug("Publishing event '%s'" % event)
+        self.__logger.debug("Publishing event '%s'", event)
         self.__configurator.handle(event)
         return event
 
@@ -1298,7 +1338,7 @@ class Listener(Participant):
 
         @param scope: The scope of the channel in which the new
                       listener should participate.
-        @type scope: Scope
+        @type scope: Scopeor or accepted by Scope constructor
         @param configurator: An in route configurator to manage the
                              receiving of events from in connectors
                              and their filtering and dispatching.
@@ -1324,7 +1364,7 @@ class Listener(Participant):
                 connector.setQualityOfServiceSpec(config.getQualityOfServiceSpec())
             self.__configurator = rsb.eventprocessing.InRouteConfigurator(connectors = connectors,
                                                                           receivingStrategy = receivingStrategy)
-        self.__configurator.setScope(scope)
+        self.__configurator.setScope(self.scope)
 
         self.__activate()
 
@@ -1444,9 +1484,10 @@ def createListener(scope, config = None):
     Creates a new Listener for the specified scope.
 
     @param scope: the scope of the new Listener. Can be a Scope object or a string.
+    @type scope: Scope or accepted by Scope constructor
     @return: a new Listener object.
     """
-    return Listener(Scope.ensureScope(scope), config)
+    return Listener(scope, config)
 
 def createInformer(scope, config=None, dataType=object):
     """
@@ -1454,17 +1495,19 @@ def createInformer(scope, config=None, dataType=object):
 
     @param scope: The scope of the new Informer. Can be a Scope object
                   or a string.
+    @type scope: Scope or accepted by Scope constructor
     @param dataType: the string representation of the data type used
                      to select converters
     @return: a new Informer object.
     """
-    return Informer(Scope.ensureScope(scope), dataType, config)
+    return Informer(scope, dataType, config)
 
 def createService(scope):
     """
     Creates a Service object operating on the given scope.
     @param scope: parent-scope of the new service. Can be a Scope
                   object or a string.
+    @type scope: Scope or accepted by Scope constructor
     @return: new Service object
     """
     raise RuntimeError, "not implemented"
@@ -1481,7 +1524,7 @@ def createServer(scope, object = None, expose = None, methods = None,
 
     @param scope: The scope under which the newly created server
                   should expose its methods.
-    @type scope: Scope
+    @type scope: Scope or accepted by Scope constructor
     @param config: The transport configuration that should be used
                    for communication performed by this server.
     @type config: ParticipantConfig
@@ -1525,7 +1568,7 @@ def createRemoteServer(scope, config = None):
 
     @param scope: The scope under which the remote server provides its
                   methods.
-    @type scope: Scope
+    @type scope: Scope or accepted by Scope constructor
     @return: A newly created L{RemoteServer} object.
     @param config: The transport configuration that should be used
                    for communication performed by this server.
